package example.org.spacciodescans;

import android.content.Intent;
import android.support.v4.app.FragmentManager;
import android.content.Context;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.support.design.widget.NavigationView;
import android.support.v4.app.Fragment;
import android.support.v4.view.GravityCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.support.v7.widget.Toolbar;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;

public class MainActivity extends AppCompatActivity{

    private DrawerLayout drawer;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);


        drawer = (DrawerLayout) findViewById(R.id.drawer_layout);

        ActionBarDrawerToggle toggle = new ActionBarDrawerToggle(
                this, drawer, toolbar, R.string.navigation_drawer_open, R.string.navigation_drawer_close);
        drawer.setDrawerListener(toggle);
        toggle.syncState();

        NavigationView navigationView = (NavigationView) findViewById(R.id.nav_view);
        if (navigationView != null) {
            getDrawerReady(navigationView);
            seleccionarItem(navigationView.getMenu().getItem(0));
        }

        drawer.openDrawer(GravityCompat.START);

        if(isConnected()) startBBDD();
        //thread para detener el progressbar al cabo de 10 segundos
        Thread t = new Thread(){
            public void run(){
                try {
                    sleep(10000);
                    runThread();
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        };
        t.start();
    }

    private void runThread() {
        runOnUiThread(new Thread(new Runnable() {
            @Override
            public void run() {
                findViewById(R.id.tpb_progressBar).setVisibility(View.GONE);
            }
        }));
    }

    private void getDrawerReady(NavigationView navigationView) {
        navigationView.setNavigationItemSelectedListener(
                new NavigationView.OnNavigationItemSelectedListener() {
                    @Override
                    public boolean onNavigationItemSelected(MenuItem menuItem) {
                        menuItem.setChecked(true);
                        seleccionarItem(menuItem);
                        drawer.closeDrawers();
                        return true;
                    }
                });
    }

    @Override
    public void onBackPressed() {
        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        if (drawer.isDrawerOpen(GravityCompat.START)) {
            drawer.closeDrawer(GravityCompat.START);
        } else {
            super.onBackPressed();
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                drawer.openDrawer(GravityCompat.START);
                return true;
            case R.id.acercaDe:
                lanzarAcercaDe(null);
                return true;

        }
        return super.onOptionsItemSelected(item);
    }


    private void seleccionarItem(MenuItem item) {

        Fragment fragmentAux = null;
        FragmentManager fragmentManager = getSupportFragmentManager();

        int id = item.getItemId();

        if (id == R.id.nav_home) {
            fragmentAux = new FragmentHome();
        } else if (id == R.id.nav_categ) {
            fragmentAux = new FragmentCategory();
        } else if (id == R.id.nav_ofer) {
            fragmentAux = new FragmentOferta();
        } else if (id == R.id.nav_about) {
            fragmentAux = new FragmentWhere();
        } else if (id == R.id.nav_contact){
            fragmentAux = new FragmentContact();
        }

        if(fragmentAux!=null){
            fragmentManager.beginTransaction().replace(R.id.contenedor_principal, fragmentAux).commit();
        }

        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        drawer.closeDrawer(GravityCompat.START);
        setTitle(item.getTitle());
    }
    /*comprueba si hay conexión a internet*/
    private boolean isConnected() {
        ConnectivityManager manager = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo info = manager.getActiveNetworkInfo();
        return info != null && info.isConnected();
    }

    //nuevo hilo para el acceso a la bbdd remota
    private void startBBDD() {
        Thread t = new Thread(new Runnable() {
            @Override
            public void run() {
                AccesoBaseDatosRemota accesoBaseDatosRemota = new AccesoBaseDatosRemota(getBaseContext());
                accesoBaseDatosRemota.accederBBDD();
            }
        });
        t.start();
    }

    public void lanzarAcercaDe(View view){
        Intent i = new Intent(this, AcercaDe.class);
        startActivity(i);
    }
}
