package example.org.spacciodescans;

import android.content.ContentProvider;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Context;
import android.content.UriMatcher;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.database.sqlite.SQLiteQueryBuilder;
import android.net.Uri;
import android.support.annotation.NonNull;
import android.text.TextUtils;
import android.util.Log;


/**
 * Created by kowalsky on 6/11/15.
 */
public class ProveedorContenido extends ContentProvider {

    public static final Uri CONTENT_URI = Uri.parse("content://example.org.proveedorContenido/productos");


    public static final String COL_ID = "_id";
    public static final String COL_NOMBRE = "nombre";
    public static final String COL_DESCRIPCION = "descripcion";
    public static final String COL_PRECIO = "precio";
    public static final String COL_OFERTA = "oferta";
    public static final String COL_CATEGORIA = "categoria";
    public static final String COL_TEXTO = "texto";
    public static final String COL_PHOTO = "photo";

    ProductosDataBaseHelper dbHelper;

    private static final int TODO = 1;
    private static final int UN_PRODUCTO = 2;

    private static final UriMatcher uriMatcher;

    static {
        uriMatcher = new UriMatcher(UriMatcher.NO_MATCH);
        uriMatcher.addURI("example.org.proveedorContenido", "productos", TODO);
        uriMatcher.addURI("example.org.proveedorContenido", "productos/#", UN_PRODUCTO);
    }

    @Override
    public boolean onCreate() {
        Context context = getContext();
        dbHelper = new ProductosDataBaseHelper(context, ProductosDataBaseHelper.DATABASE_NAME, null, ProductosDataBaseHelper.DATABASE_VERSION);
        return true;
    }

    public Cursor listado(){
        SQLiteDatabase bd = dbHelper.getReadableDatabase();
        return bd.rawQuery("SELECT * FROM products", null);
    }

    @Override
    public Cursor query(@NonNull Uri uri, String[] projection, String selection, String[] selectionArgs, String sortOrder) {

        SQLiteDatabase bdAux;

        bdAux = dbHelper.getWritableDatabase();

        SQLiteQueryBuilder queryBuilder = new SQLiteQueryBuilder();

        switch (uriMatcher.match(uri)){
            case UN_PRODUCTO:
                queryBuilder.appendWhere(COL_ID + "=" + uri.getPathSegments().get(1));
                break;
            default: break;
        }


        String orderBy;
        if (TextUtils.isEmpty(sortOrder)) {
            orderBy = COL_NOMBRE;
        } else {
            orderBy = sortOrder;
        }

        queryBuilder.setTables(ProductosDataBaseHelper.TABLA_PRODUCTOS);
        // Se ejecuta la consulta
        Cursor cursor = queryBuilder.query(bdAux,
                projection,
                selection, selectionArgs,
                null, null,
                orderBy);

        cursor.setNotificationUri(getContext().getContentResolver(), uri);
        return cursor;
    }

    @Override
    public String getType(@NonNull Uri uri) {
        switch (uriMatcher.match(uri)) {
            case TODO  : return "vnd.android.cursor.dir/vnd.android.productos";
            case UN_PRODUCTO: return "vnd.android.cursor.item/vnd.android.productos";
            default: throw new IllegalArgumentException("URI no prevista: " + uri);
        }
    }

    @Override
    public Uri insert(@NonNull Uri uri, ContentValues values) {
        SQLiteDatabase bDatos = dbHelper.getWritableDatabase();
        long rowID = bDatos.insert(ProductosDataBaseHelper.TABLA_PRODUCTOS, "description", values);
        if(rowID > 0) {
            Uri uri1 = ContentUris.withAppendedId(CONTENT_URI, rowID);
            getContext().getContentResolver().notifyChange(uri, null);
        }
        return uri;
    }

    @Override
    public int delete(@NonNull Uri uri, String selection, String[] selectionArgs) {
        SQLiteDatabase bDatos = dbHelper.getWritableDatabase();
        int qFilas;

        switch (uriMatcher.match(uri)){
            case TODO:
                qFilas = bDatos.delete(ProductosDataBaseHelper.TABLA_PRODUCTOS, selection, selectionArgs);
                break;
            case UN_PRODUCTO:
                String segment = uri.getPathSegments().get(1);
                //si existe, elimina la fila
                qFilas = bDatos.delete(ProductosDataBaseHelper.TABLA_PRODUCTOS, COL_ID + "=" + segment + (!TextUtils.isEmpty(selection) ? " AND ("
                        + selection + ')' : ""), selectionArgs);
                break;
            default: throw new IllegalArgumentException("Uri no prevista: " + uri);
        }
        getContext().getContentResolver().notifyChange(uri, null);
        // Devuelve la cantidad de filas eliminadas
        return qFilas;
    }


    @Override
    public int update(@NonNull Uri uri, ContentValues values, String selection, String[] selectionArgs) {
            SQLiteDatabase bDatos = dbHelper.getWritableDatabase();
            int qFilas;

            switch (uriMatcher.match(uri)) {
                case TODO:
                    qFilas = bDatos.update(ProductosDataBaseHelper.TABLA_PRODUCTOS, values, selection, selectionArgs);
                    break;
                case UN_PRODUCTO:
                    String segment = uri.getPathSegments().get(1);
                    qFilas = bDatos.update(ProductosDataBaseHelper.TABLA_PRODUCTOS,
                            values, COL_ID
                                    + "=" + segment
                                    + (!TextUtils.isEmpty(selection) ? " AND ("
                                    + selection + ')' : ""), selectionArgs);
                    break;
                default: throw new IllegalArgumentException("URI no prevista: " + uri);
            }

            getContext().getContentResolver().notifyChange(uri, null);
            return qFilas;
    }

    private static class ProductosDataBaseHelper extends SQLiteOpenHelper {

        private static final String TAG = "LectorProveedor";

        private static final String DATABASE_NAME = "spaccioDescans.db";
        private static final int DATABASE_VERSION = 1;
        private static final String TABLA_PRODUCTOS = "productos";

        private static final String DATABASE_CREATE = "create table " + TABLA_PRODUCTOS + " ("
                        + COL_ID + " integer primary key autoincrement, "
                        + COL_NOMBRE + " TEXT, "
                        + COL_DESCRIPCION + " TEXT, "
                        + COL_PRECIO + " INTEGER, "
                        + COL_OFERTA + " TEXT, "
                        + COL_CATEGORIA + " TEXT, "
                        + COL_TEXTO + " TEXT, "
                        + COL_PHOTO + " BLOB); ";

        public ProductosDataBaseHelper(Context context, String name, SQLiteDatabase.CursorFactory factory, int version){
            super(context, name, factory, version);
        }

        @Override
        public void onCreate(SQLiteDatabase db) {
            db.execSQL(DATABASE_CREATE);
        }

        @Override
        public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
            Log.w(TAG, "Se actualiza la versión de la base de datos " + oldVersion + " a la versión " + newVersion + ", se eliminarán los datos.");
            db.execSQL("DROP TABLE IF EXISTS " + TABLA_PRODUCTOS);
            onCreate(db);
        }
    }
}
